@@ -1,6 +1,6 @@
 ;;; phps-mode-lex-analyzer.el -- Lex analyzer for PHPs -*- lexical-binding: t -*-
 
-;; Copyright (C) 2018-2022  Free Software Foundation, Inc.
+;; Copyright (C) 2018-2023  Free Software Foundation, Inc.
 
 
 ;;; Commentary:
@@ -24,8 +24,6 @@
 (require 'phps-mode-serial)
 (require 'phps-mode-syntax-color)
 (require 'phps-mode-ast)
-(require 'phps-mode-ast-bookkeeping)
-(require 'phps-mode-ast-imenu)
 
 (require 'subr-x)
 
@@ -42,6 +40,15 @@
 (defvar-local phps-mode-lex-analyzer--processed-buffer-p nil
   "Flag whether current buffer is processed or not.")
 
+(defvar-local phps-mode-lex-analyzer--error-end nil
+  "Point of error end or nil.")
+
+(defvar-local phps-mode-lex-analyzer--error-message nil
+  "Message of error.")
+
+(defvar-local phps-mode-lex-analyzer--error-start nil
+  "Point of error start or nil.")
+
 
 ;; VARIABLES
 
@@ -58,6 +65,9 @@
 (defvar-local phps-mode-lex-analyzer--bookkeeping nil
   "Bookkeeping of all variables in tokens.")
 
+(defvar-local phps-mode-lex-analyzer--symbol-table nil
+  "Symbol-table of all symbols found in parse.")
+
 (defvar-local phps-mode-lex-analyzer--tokens nil
   "Latest tokens.")
 
@@ -91,6 +101,9 @@
 (defvar-local phps-mode-lex-analyzer--lexer-max-index nil
   "Max-index of lex-analyzer.")
 
+(defvar phps-mode-lex-analyzer--show-profiling-information nil
+  "Show profiling information.")
+
 
 ;; FUNCTIONS
 
@@ -157,9 +170,9 @@
     (cond
 
      ((when bookkeeping-value
-        (if (> bookkeeping-value 0)
-            'font-lock-variable-name-face
-          'font-lock-warning-face)))
+        (if (= bookkeeping-value 0)
+            'font-lock-warning-face
+          'font-lock-variable-name-face)))
 
      ((when-let ((face
                   (gethash
@@ -202,183 +215,280 @@
            (cdr (cdr (car phps-mode-lexer--generated-new-tokens)))))))))
 
 (defun phps-mode-lex-analyzer--re2c-run (&optional force-synchronous allow-cache-read allow-cache-write)
-  "Run lexer, optionally FORCE-SYNCHRONOUS mode, ALLOW-CACHE-READ and ALLOW-CACHE-WRITE."
+  "Run lexer, optionally FORCE-SYNCHRONOUS mode,
+ALLOW-CACHE-READ and ALLOW-CACHE-WRITE."
   (interactive)
   (require 'phps-mode-macros)
   (phps-mode-debug-message (message "Lexer run"))
 
-  (let ((buffer-name (buffer-name))
-        (buffer-contents (buffer-substring-no-properties (point-min) (point-max)))
-        (async (and (boundp 'phps-mode-async-process)
-                    phps-mode-async-process))
-        (async-by-process (and (boundp 'phps-mode-async-process-using-async-el)
-                               phps-mode-async-process-using-async-el)))
-    (when force-synchronous
-      (setq async nil))
+  (let ((buffer-contents))
+    (save-restriction
+      (widen)
+      (setq
+       buffer-contents
+       (buffer-substring-no-properties
+        (point-min)
+        (point-max))))
+    (let ((buffer-name (buffer-name))
+          (async (and (boundp 'phps-mode-async-process)
+                      phps-mode-async-process))
+          (async-by-process (and (boundp 'phps-mode-async-process-using-async-el)
+                                 phps-mode-async-process-using-async-el))
+          (timer-start)
+          (timer-elapsed-lexer)
+          (timer-elapsed-parser)
+          (timer-start-syntax-coloring)
+          (timer-finished-syntax-coloring)
+          (timer-elapsed-syntax-coloring)
+          (timer-elapsed)
+          (timer-end)
+          (narrowed-point-min (point-min))
+          (narrowed-point-max (point-max)))
+      (when force-synchronous
+        (setq async nil))
+      (let ((current-time (current-time)))
+        (setq
+         timer-start
+         (+
+          (car current-time)
+          (car (cdr current-time))
+          (* (car (cdr (cdr current-time))) 0.000001))))
 
-    (phps-mode-serial-commands
-     buffer-name
+      (phps-mode-serial-commands
 
-     (lambda()
-       (phps-mode-lex-analyzer--lex-string
-        buffer-contents
-        nil
-        nil
-        nil
-        nil
-        nil
-        nil
-        nil
-        nil
-        nil
-        buffer-file-name
-        allow-cache-read
-        allow-cache-write))
+       buffer-name
 
-     (lambda(lex-result)
-       (when (get-buffer buffer-name)
-         (with-current-buffer buffer-name
+       ;; Potentially asynchronous thread
+       (lambda()
+         (phps-mode-lex-analyzer--lex-string
+          buffer-contents
+          nil
+          nil
+          nil
+          nil
+          nil
+          nil
+          nil
+          nil
+          nil
+          buffer-file-name
+          allow-cache-read
+          allow-cache-write))
 
-           ;; Move variables into this buffers local variables
-           (setq phps-mode-lex-analyzer--tokens (nth 0 lex-result))
-           (setq phps-mode-lex-analyzer--states (nth 1 lex-result))
-           (setq phps-mode-lex-analyzer--state (nth 2 lex-result))
-           (setq phps-mode-lex-analyzer--state-stack (nth 3 lex-result))
-           (setq phps-mode-lex-analyzer--heredoc-label (nth 4 lex-result))
-           (setq phps-mode-lex-analyzer--heredoc-label-stack (nth 5 lex-result))
-           (setq phps-mode-lex-analyzer--nest-location-stack (nth 6 lex-result))
-           (setq phps-mode-lex-analyzer--parse-trail (nth 7 lex-result))
-           (setq phps-mode-lex-analyzer--parse-error (nth 8 lex-result))
-           (setq phps-mode-lex-analyzer--ast (nth 9 lex-result))
+       ;; Main thread - successful operation
+       (lambda(lex-result)
+         (when (get-buffer buffer-name)
+           (with-current-buffer buffer-name
 
-           ;; Catch errors in bookkeeping generation
-           (condition-case conditions
-               (phps-mode-ast-bookkeeping--generate
-                phps-mode-lex-analyzer--ast)
-             (error
-              (display-warning
-               'phps-mode
-               (format "Failed to generate bookkeeping: %S" conditions)
-               :warning
-               "*PHPs Bookkeeping Generation Errors*")))
-           (setq phps-mode-lex-analyzer--bookkeeping
-                 phps-mode-ast-bookkeeping--index)
+             ;; Move variables into this buffers local variables
+             (setq phps-mode-lex-analyzer--tokens (nth 0 lex-result))
+             (setq phps-mode-lex-analyzer--states (nth 1 lex-result))
+             (setq phps-mode-lex-analyzer--state (nth 2 lex-result))
+             (setq phps-mode-lex-analyzer--state-stack (nth 3 lex-result))
+             (setq phps-mode-lex-analyzer--heredoc-label (nth 4 lex-result))
+             (setq phps-mode-lex-analyzer--heredoc-label-stack (nth 5 lex-result))
+             (setq phps-mode-lex-analyzer--nest-location-stack (nth 6 lex-result))
+             (setq phps-mode-lex-analyzer--parse-trail (nth 7 lex-result))
+             (setq phps-mode-lex-analyzer--parse-error (nth 8 lex-result))
+             (setq phps-mode-lex-analyzer--ast (nth 9 lex-result))
+             (setq phps-mode-lex-analyzer--bookkeeping (nth 10 lex-result))
+             (setq phps-mode-lex-analyzer--imenu (nth 11 lex-result))
+             (setq phps-mode-lex-analyzer--symbol-table (nth 12 lex-result))
+             (setq timer-elapsed-lexer (nth 13 lex-result))
+             (setq timer-elapsed-parser (nth 14 lex-result))
 
-           ;; Catch errors in imenu generation
-           (condition-case conditions
-               (phps-mode-ast-imenu--generate
-                phps-mode-lex-analyzer--ast)
-             (error
-              (display-warning
-               'phps-mode
-               (format "Failed to generate imenu: %S" conditions)
-               :warning
-               "*PHPs Imenu Generation Errors*")))
-           (setq phps-mode-lex-analyzer--imenu phps-mode-ast-imenu--index)
+             (setq phps-mode-lex-analyzer--processed-buffer-p t)
+             (phps-mode-lex-analyzer--reset-imenu)
 
-           (setq phps-mode-lex-analyzer--processed-buffer-p t)
-           (phps-mode-lex-analyzer--reset-imenu)
+             (let ((current-time (current-time)))
+               (setq
+                timer-start-syntax-coloring
+                (+
+                 (car current-time)
+                 (car (cdr current-time))
+                 (* (car (cdr (cdr current-time))) 0.000001))))
 
-           ;; Apply syntax color on tokens
-           (dolist (token phps-mode-lex-analyzer--tokens)
-             (let ((start (car (cdr token)))
-                   (end (cdr (cdr token))))
-               (let ((token-syntax-color
-                      (phps-mode-lex-analyzer--get-token-syntax-color token)))
-                 (if token-syntax-color
-                     (phps-mode-lex-analyzer--set-region-syntax-color
-                      start end (list 'font-lock-face token-syntax-color))
-                   (phps-mode-lex-analyzer--clear-region-syntax-color
-                    start end)))))
+             ;; Apply syntax color
+             (phps-mode-lex-analyzer--clear-region-syntax-color
+              (point-min)
+              (point-max))
+             (dolist (token phps-mode-lex-analyzer--tokens)
+               (let ((start (car (cdr token)))
+                     (end (cdr (cdr token))))
+                 (when (and
+                        (>= start narrowed-point-min)
+                        (<= end narrowed-point-max))
+                   (let ((token-syntax-color
+                          (phps-mode-lex-analyzer--get-token-syntax-color token)))
+                     (when token-syntax-color
+                       (phps-mode-lex-analyzer--set-region-syntax-color
+                        start
+                        end
+                        (list 'font-lock-face token-syntax-color)))))))
 
-           ;; Reset buffer changes minimum index
-           (phps-mode-lex-analyzer--reset-changes)
+             (let ((current-time (current-time)))
+               (setq
+                timer-finished-syntax-coloring
+                (+
+                 (car current-time)
+                 (car (cdr current-time))
+                 (* (car (cdr (cdr current-time))) 0.000001)))
+               (setq
+                timer-elapsed-syntax-coloring
+                (-
+                 timer-finished-syntax-coloring
+                 timer-start-syntax-coloring)))
 
-           ;; Signal parser error (if any)
-           (when phps-mode-lex-analyzer--parse-error
+             ;; Reset buffer changes minimum index
+             (phps-mode-lex-analyzer--reset-changes)
 
-             ;; Paint error
-             (phps-mode-lex-analyzer--set-region-syntax-color
-              (nth 4 phps-mode-lex-analyzer--parse-error)
-              (point-max)
-              (list 'font-lock-face 'font-lock-warning-face))
+             ;; Signal parser error (if any)
+             (if phps-mode-lex-analyzer--parse-error
+                 (progn
 
-             ;; Display error
-             (display-warning
-              'phps-mode
-              (nth 1 phps-mode-lex-analyzer--parse-error)
-              :warning
-              "*PHPs Parser Errors*")
+                   ;; Paint error
+                   (phps-mode-lex-analyzer--set-region-syntax-color
+                    (nth 4 phps-mode-lex-analyzer--parse-error)
+                    (point-max)
+                    (list 'font-lock-face 'font-lock-warning-face))
 
-             ;; Signal that causes updated mode-line status
-             (signal
-              'phps-parser-error
-              (list
-               (nth 1 phps-mode-lex-analyzer--parse-error)
-               (nth 4 phps-mode-lex-analyzer--parse-error))))
+                   ;; Set error
+                   (setq phps-mode-lex-analyzer--error-end nil)
+                   (setq phps-mode-lex-analyzer--error-message (nth 1 phps-mode-lex-analyzer--parse-error))
+                   (setq phps-mode-lex-analyzer--error-start (nth 4 phps-mode-lex-analyzer--parse-error))
 
-           )))
+                   ;; Signal that causes updated mode-line status
+                   (signal
+                    'phps-parser-error
+                    (list
+                     (nth 1 phps-mode-lex-analyzer--parse-error)
+                     (nth 4 phps-mode-lex-analyzer--parse-error))))
 
-     (lambda(result)
-       (when (get-buffer buffer-name)
-         (with-current-buffer buffer-name
-           (let ((error-type (nth 0 result))
-                 (error-message (nth 1 result))
-                 (error-start (nth 2 result))
-                 (error-end (nth 3 result)))
-             (phps-mode-lex-analyzer--reset-local-variables)
+               ;; Reset error
+               (setq phps-mode-lex-analyzer--error-end nil)
+               (setq phps-mode-lex-analyzer--error-message nil)
+               (setq phps-mode-lex-analyzer--error-start nil))))
 
-             (when error-message
-               (cond
+         (let ((current-time (current-time)))
+           (setq
+            timer-end
+            (+
+             (car current-time)
+             (car (cdr current-time))
+             (* (car (cdr (cdr current-time))) 0.000001)))
+           (setq
+            timer-elapsed
+            (-
+             timer-end
+             timer-start)))
 
-                ((equal error-type 'phps-lexer-error)
-                 (when error-start
-                   (if error-end
+         (when phps-mode-lex-analyzer--show-profiling-information
+           (let ((lexer-percentage
+                  (/ timer-elapsed-lexer timer-elapsed))
+                 (parser-percentage
+                  (/ timer-elapsed-parser timer-elapsed))
+                 (syntax-coloring-percentage
+                  (/ timer-elapsed-syntax-coloring timer-elapsed)))
+             (message
+              "Total: %d.2s (lex: %d.2%%, parse: %d.2%%, color: %d.2%%)"
+              timer-elapsed
+              lexer-percentage
+              parser-percentage
+              syntax-coloring-percentage)))
+
+         )
+
+       ;; Main thread - error handling
+       (lambda(result)
+         (when (get-buffer buffer-name)
+           (with-current-buffer buffer-name
+             (let ((error-type (nth 0 result))
+                   (error-message (nth 1 result))
+                   (error-start (nth 2 result))
+                   (error-end (nth 3 result)))
+               (phps-mode-lex-analyzer--reset-local-variables)
+
+               (when error-message
+                 (cond
+
+                  ((equal error-type 'phps-lexer-error)
+                   (when error-start
+                     (if error-end
+                         (phps-mode-lex-analyzer--set-region-syntax-color
+                          error-start
+                          error-end
+                          (list 'font-lock-face 'font-lock-warning-face))
                        (phps-mode-lex-analyzer--set-region-syntax-color
                         error-start
-                        error-end
-                        (list 'font-lock-face 'font-lock-warning-face))
-                     (phps-mode-lex-analyzer--set-region-syntax-color
-                      error-start
-                      (point-max)
-                      (list 'font-lock-face 'font-lock-warning-face))))
-                 (display-warning
-                  'phps-mode
-                  error-message
-                  :warning
-                  "*PHPs Lexer Errors*"))
+                        (point-max)
+                        (list 'font-lock-face 'font-lock-warning-face)))
 
-                (t
-                 (display-warning
-                  error-type
-                  error-message
-                  :warning))
+                     ;; Set error
+                     (setq phps-mode-lex-analyzer--error-end (if error-end error-end nil))
+                     (setq phps-mode-lex-analyzer--error-message error-message)
+                     (setq phps-mode-lex-analyzer--error-start error-start)))
 
-                )
+                  (t
+                   (progn
 
-               )
+                     ;; Reset error
+                     (setq phps-mode-lex-analyzer--error-end nil)
+                     (setq phps-mode-lex-analyzer--error-message nil)
+                     (setq phps-mode-lex-analyzer--error-start nil)
 
-             ))))
+                     (display-warning
+                      error-type
+                      error-message
+                      :warning)))))))))
 
-     nil
-     async
-     async-by-process)))
+       nil
+       async
+       async-by-process))))
 
 (defun phps-mode-lex-analyzer--incremental-lex-string
-    (buffer-name buffer-contents incremental-start-new-buffer point-max
-                 head-states incremental-state incremental-state-stack incremental-heredoc-label incremental-heredoc-label-stack incremental-nest-location-stack head-tokens &optional force-synchronous filename allow-cache-write)
+    (buffer-name
+     buffer-contents
+     incremental-start-new-buffer
+     point-max
+     head-states
+     incremental-state
+     incremental-state-stack
+     incremental-heredoc-label
+     incremental-heredoc-label-stack
+     incremental-nest-location-stack
+     head-tokens
+     &optional
+     force-synchronous
+     filename
+     allow-cache-write)
   "Incremental lex region."
-  (let ((async (and (boundp 'phps-mode-async-process)
+  (let* ((async (and (boundp 'phps-mode-async-process)
                     phps-mode-async-process))
         (async-by-process (and (boundp 'phps-mode-async-process-using-async-el)
-                               phps-mode-async-process-using-async-el)))
+                               phps-mode-async-process-using-async-el))
+        (timer-start)
+        (timer-elapsed-lexer)
+        (timer-elapsed-parser)
+        (timer-start-syntax-coloring)
+        (timer-finished-syntax-coloring)
+        (timer-elapsed-syntax-coloring)
+        (timer-elapsed)
+        (timer-end))
     (when force-synchronous
       (setq async nil))
+    (let ((current-time (current-time)))
+      (setq
+       timer-start
+       (+
+        (car current-time)
+        (car (cdr current-time))
+        (* (car (cdr (cdr current-time))) 0.000001))))
 
     (phps-mode-serial-commands
 
      buffer-name
 
+     ;; Potentially asynchronous thread
      (lambda()
        (phps-mode-lex-analyzer--lex-string
         buffer-contents
@@ -395,13 +505,11 @@
         nil
         allow-cache-write))
 
+     ;; Main thread - successful operation
      (lambda(lex-result)
        (when (get-buffer buffer-name)
          (with-current-buffer buffer-name
 
-           (phps-mode-debug-message
-            (message "Incrementally-lexed-string: %s" result))
-
            (setq phps-mode-lex-analyzer--tokens (nth 0 lex-result))
            (setq phps-mode-lex-analyzer--states (nth 1 lex-result))
            (setq phps-mode-lex-analyzer--state (nth 2 lex-result))
@@ -412,76 +520,121 @@
            (setq phps-mode-lex-analyzer--parse-trail (nth 7 lex-result))
            (setq phps-mode-lex-analyzer--parse-error (nth 8 lex-result))
            (setq phps-mode-lex-analyzer--ast (nth 9 lex-result))
-
-           ;; Catch errors in bookkeeping generation
-           (condition-case conditions
-               (phps-mode-ast-bookkeeping--generate
-                phps-mode-lex-analyzer--ast)
-             (error
-              (display-warning
-               'phps-mode
-               (format "Failed to generate bookkeeping: %S" conditions)
-               :warning
-               "*PHPs Bookkeeping Generation Errors*")))
-           (setq phps-mode-lex-analyzer--bookkeeping
-                 phps-mode-ast-bookkeeping--index)
-
-           ;; Catch errors in imenu generation
-           (condition-case conditions
-               (phps-mode-ast-imenu--generate
-                phps-mode-lex-analyzer--ast)
-             (error
-              (display-warning
-               'phps-mode
-               (format "Failed to generate imenu: %S" conditions)
-               :warning
-               "*PHPs Imenu Generation Errors*")))
-           (setq phps-mode-lex-analyzer--imenu phps-mode-ast-imenu--index)
+           (setq phps-mode-lex-analyzer--bookkeeping (nth 10 lex-result))
+           (setq phps-mode-lex-analyzer--imenu (nth 11 lex-result))
+           (setq phps-mode-lex-analyzer--symbol-table (nth 12 lex-result))
+           (setq timer-elapsed-lexer (nth 13 lex-result))
+           (setq timer-elapsed-parser (nth 14 lex-result))
 
            (phps-mode-debug-message
-            (message "Incremental tokens: %s" phps-mode-lex-analyzer--tokens))
+            (message
+             "Incremental tokens: %s"
+             phps-mode-lex-analyzer--tokens))
 
            ;; Save processed result
            (setq phps-mode-lex-analyzer--processed-buffer-p t)
            (phps-mode-lex-analyzer--reset-imenu)
 
+           (let ((current-time (current-time)))
+             (setq
+              timer-start-syntax-coloring
+              (+
+               (car current-time)
+               (car (cdr current-time))
+               (* (car (cdr (cdr current-time))) 0.000001))))
+
            ;; Apply syntax color on tokens
+           (phps-mode-lex-analyzer--clear-region-syntax-color
+            incremental-start-new-buffer
+            point-max)
            (dolist (token phps-mode-lex-analyzer--tokens)
              (let ((start (car (cdr token)))
                    (end (cdr (cdr token))))
+               (when (and
+                      (>= start incremental-start-new-buffer)
+                      (<= end point-max))
 
                ;; Apply syntax color on token
                (let ((token-syntax-color (phps-mode-lex-analyzer--get-token-syntax-color token)))
-                 (if token-syntax-color
-                     (phps-mode-lex-analyzer--set-region-syntax-color start end (list 'font-lock-face token-syntax-color))
-                   (phps-mode-lex-analyzer--clear-region-syntax-color start end)))))
+                 (when token-syntax-color
+                   (phps-mode-lex-analyzer--set-region-syntax-color
+                    start
+                    end
+                    (list 'font-lock-face token-syntax-color)))))))
+
+           (let ((current-time (current-time)))
+             (setq
+              timer-finished-syntax-coloring
+              (+
+               (car current-time)
+               (car (cdr current-time))
+               (* (car (cdr (cdr current-time))) 0.000001)))
+             (setq
+              timer-elapsed-syntax-coloring
+              (-
+               timer-finished-syntax-coloring
+               timer-start-syntax-coloring)))
 
            ;; Reset buffer changes minimum index
            (phps-mode-lex-analyzer--reset-changes)
 
            ;; Signal parser error (if any)
-           (when phps-mode-lex-analyzer--parse-error
+           (if phps-mode-lex-analyzer--parse-error
+               (progn
 
-             ;; Paint error
-             (phps-mode-lex-analyzer--set-region-syntax-color
-              (nth 4 phps-mode-lex-analyzer--parse-error)
-              (point-max)
-              (list 'font-lock-face 'font-lock-warning-face))
+                 ;; Paint error
+                 (phps-mode-lex-analyzer--set-region-syntax-color
+                  (nth 4 phps-mode-lex-analyzer--parse-error)
+                  (point-max)
+                  (list 'font-lock-face 'font-lock-warning-face))
 
-             ;; Display error
-             (display-warning
-              'phps-mode
-              (nth 1 phps-mode-lex-analyzer--parse-error)
-              :warning
-              "*PHPs Parser Errors*")
+                 ;; Set error
+                 (setq phps-mode-lex-analyzer--error-end nil)
+                 (setq phps-mode-lex-analyzer--error-message (nth 1 phps-mode-lex-analyzer--parse-error))
+                 (setq phps-mode-lex-analyzer--error-start (nth 4 phps-mode-lex-analyzer--parse-error))
 
-             ;; Signal that causes updated mode-line status
-             (signal
-              'phps-parser-error
-              (list
-               (nth 1 phps-mode-lex-analyzer--parse-error)
-               (nth 4 phps-mode-lex-analyzer--parse-error)))))))
+                 ;; Signal that causes updated mode-line status
+                 (signal
+                  'phps-parser-error
+                  (list
+                   (nth 1 phps-mode-lex-analyzer--parse-error)
+                   (nth 4 phps-mode-lex-analyzer--parse-error))))
 
+             ;; Reset error
+             (setq phps-mode-lex-analyzer--error-end nil)
+             (setq phps-mode-lex-analyzer--error-message nil)
+             (setq phps-mode-lex-analyzer--error-start nil))))
+
+           (let ((current-time (current-time)))
+             (setq
+              timer-end
+              (+
+               (car current-time)
+               (car (cdr current-time))
+               (* (car (cdr (cdr current-time))) 0.000001)))
+             (setq
+              timer-elapsed
+              (-
+               timer-end
+               timer-start)))
+
+           (when phps-mode-lex-analyzer--show-profiling-information
+             (let ((lexer-percentage
+                    (/ timer-elapsed-lexer timer-elapsed))
+                   (parser-percentage
+                    (/ timer-elapsed-parser timer-elapsed))
+                   (syntax-coloring-percentage
+                    (/ timer-elapsed-syntax-coloring timer-elapsed)))
+               (message
+                "Total: %d.2s (lex: %d.2%%, parse: %d.2%%, color: %d.2%%)"
+                timer-elapsed
+                lexer-percentage
+                parser-percentage
+               syntax-coloring-percentage)))
+
+       )
+
+     ;; Main thread - error handling
      (lambda(result)
        (when (get-buffer buffer-name)
          (with-current-buffer buffer-name
@@ -506,17 +659,24 @@
                       error-start
                       (point-max)
                       (list 'font-lock-face 'font-lock-warning-face))))
-                 (display-warning
-                  'phps-mode
-                  error-message
-                  :warning
-                  "*PHPs Lexer Errors*"))
+
+                 ;; Set error
+                 (setq phps-mode-lex-analyzer--error-end (if error-end error-end nil))
+                 (setq phps-mode-lex-analyzer--error-message error-message)
+                 (setq phps-mode-lex-analyzer--error-start error-start))
 
                 (t
-                 (display-warning
-                  error-type
-                  error-message
-                  :warning))))))))
+                 (progn
+
+                 ;; Reset error
+                 (setq phps-mode-lex-analyzer--error-end nil)
+                 (setq phps-mode-lex-analyzer--error-message nil)
+                 (setq phps-mode-lex-analyzer--error-start nil)
+
+                   (display-warning
+                    error-type
+                    error-message
+                    :warning)))))))))
 
      nil
      async
@@ -577,7 +737,8 @@
     new-states))
 
 (defun phps-mode-lex-analyzer--move-tokens (start diff)
-  "Update tokens with moved lexer tokens after or equal to START with modification DIFF."
+  "Update tokens with moved lexer tokens after
+or equal to START with modification DIFF."
   (when phps-mode-lex-analyzer--tokens
     (setq
      phps-mode-lex-analyzer--tokens
@@ -587,7 +748,8 @@
       diff))))
 
 (defun phps-mode-lex-analyzer--get-moved-tokens (old-tokens start diff)
-  "Return moved lexer OLD-TOKENS positions after (or equal to) START with DIFF points."
+  "Return moved lexer OLD-TOKENS positions after
+(or equal to) START with DIFF points."
   (let ((new-tokens '()))
     (when old-tokens
 
@@ -611,7 +773,8 @@
   (setq phps-mode-lex-analyzer--change-min nil))
 
 (defun phps-mode-lex-analyzer--process-changes (&optional buffer force-synchronous)
-  "Run incremental lexer on BUFFER.  Return list of performed operations.  Optionally do it FORCE-SYNCHRONOUS."
+  "Run incremental lexer on BUFFER.  Return list
+of performed operations.  Optionally do it FORCE-SYNCHRONOUS."
   (unless buffer
     (setq buffer (current-buffer)))
   (phps-mode-debug-message
@@ -633,7 +796,6 @@
                     (incremental-heredoc-label nil)
                     (incremental-heredoc-label-stack nil)
                     (incremental-nest-location-stack nil)
-                    (incremental-tokens nil)
                     (head-states '())
                     (head-tokens '())
                     (change-start phps-mode-lex-analyzer--change-min)
@@ -724,24 +886,33 @@
 
                             ;; Do partial lex from previous-token-end to change-stop
 
-                            (phps-mode-lex-analyzer--incremental-lex-string
-                             (buffer-name)
-                             (buffer-substring-no-properties (point-min) (point-max))
-                             incremental-start-new-buffer
-                             (point-max)
-                             head-states
-                             incremental-state
-                             incremental-state-stack
-                             incremental-heredoc-label
-                             incremental-heredoc-label-stack
-                             incremental-nest-location-stack
-                             head-tokens
-                             force-synchronous
-                             (if (buffer-modified-p) nil buffer-file-name)
-                             (not (buffer-modified-p)))
-
-                            (phps-mode-debug-message
-                             (message "Incremental tokens: %s" incremental-tokens)))
+                            (let ((buffer-contents)
+                                  (buffer-max))
+                              (save-restriction
+                                (widen)
+                                (setq
+                                 buffer-contents
+                                 (buffer-substring-no-properties
+                                  (point-min)
+                                  (point-max)))
+                                (setq
+                                 buffer-max
+                                 (point-max)))
+                              (phps-mode-lex-analyzer--incremental-lex-string
+                               (buffer-name)
+                               buffer-contents
+                               incremental-start-new-buffer
+                               buffer-max
+                               head-states
+                               incremental-state
+                               incremental-state-stack
+                               incremental-heredoc-label
+                               incremental-heredoc-label-stack
+                               incremental-nest-location-stack
+                               head-tokens
+                               force-synchronous
+                               (if (buffer-modified-p) nil buffer-file-name)
+                               (not (buffer-modified-p)))))
 
                         (push (list 'FOUND-NO-HEAD-STATES incremental-start-new-buffer) log)
                         (phps-mode-debug-message
@@ -1043,14 +1214,18 @@
                   (cond
 
                    ((string=
-                     (buffer-substring-no-properties offset-comment-start (+ offset-comment-start 1))
+                     (buffer-substring-no-properties
+                      offset-comment-start
+                      (+ offset-comment-start 1))
                      "#")
                     (save-excursion
                       (goto-char offset-comment-start)
                       (delete-char 1))
                     (setq offset (- offset 1)))
                    ((string=
-                     (buffer-substring-no-properties offset-comment-start (+ offset-comment-start 2))
+                     (buffer-substring-no-properties
+                      offset-comment-start
+                      (+ offset-comment-start 2))
                      "//")
                     (save-excursion
                       (goto-char offset-comment-start)
@@ -1065,7 +1240,9 @@
                   
                   (setq offset-comment-end (+ token-end offset))
                   (if (string=
-                       (buffer-substring-no-properties (- offset-comment-end 3) offset-comment-end)
+                       (buffer-substring-no-properties
+                        (- offset-comment-end 3)
+                        offset-comment-end)
                        " */")
                       (progn
                         (phps-mode-debug-message
@@ -1136,8 +1313,24 @@
   ;; to enable nice presentation
   (require 'phps-mode-macros)
 
-  (let ((loaded-from-cache)
-        (cache-key))
+  (let* ((loaded-from-cache)
+         (cache-key)
+         (timer-start-lexer)
+         (timer-finished-lexer)
+         (timer-elapsed-lexer)
+         (timer-start-parser)
+         (timer-finished-parser)
+         (timer-elapsed-parser))
+
+    (let* ((current-time (current-time))
+           (start-time
+            (+
+             (car current-time)
+             (car (cdr current-time))
+             (* (car (cdr (cdr current-time))) 0.000001))))
+      (setq
+       timer-start-lexer
+       start-time))
 
     ;; Build cache key if possible
     (when (and
@@ -1164,7 +1357,10 @@
               (generate-new-buffer "*PHPs Lexer*"))
              (parse-error)
              (parse-trail)
-             (ast-tree))
+             (ast-tree)
+             (bookkeeping)
+             (imenu)
+             (symbol-table))
 
         ;; Create temporary buffer and run lexer in it
         (when (get-buffer buffer)
@@ -1236,6 +1432,22 @@
             (setq heredoc-label-stack phps-mode-lexer--heredoc-label-stack)
             (setq nest-location-stack phps-mode-lexer--nest-location-stack)
 
+            (let* ((current-time (current-time))
+                   (end-time
+                    (+
+                     (car current-time)
+                     (car (cdr current-time))
+                     (* (car (cdr (cdr current-time))) 0.000001))))
+              (setq
+               timer-finished-lexer
+               end-time)
+              (setq
+               timer-start-parser
+               end-time)
+              (setq
+               timer-elapsed-lexer
+               (- timer-finished-lexer timer-start-lexer)))
+
             ;; Error-free parse here
             (condition-case conditions
                 (progn
@@ -1248,6 +1460,23 @@
             ;; Need to copy buffer-local values before killing buffer
             (setq parse-trail phps-mode-ast--parse-trail)
             (setq ast-tree phps-mode-ast--tree)
+            (setq bookkeeping phps-mode-parser-sdt-bookkeeping)
+            (setq imenu phps-mode-parser-sdt-symbol-imenu)
+            (setq symbol-table phps-mode-parser-sdt-symbol-table)
+
+            (let* ((current-time
+                    (current-time))
+                   (end-time
+                    (+
+                     (car current-time)
+                     (car (cdr current-time))
+                     (* (car (cdr (cdr current-time))) 0.000001))))
+              (setq
+               timer-finished-parser
+               end-time)
+              (setq
+               timer-elapsed-parser
+               (- timer-finished-parser timer-start-parser)))
 
             (kill-buffer)))
 
@@ -1262,7 +1491,12 @@
                 nest-location-stack
                 parse-trail
                 parse-error
-                ast-tree)))
+                ast-tree
+                bookkeeping
+                imenu
+                symbol-table
+                timer-elapsed-lexer
+                timer-elapsed-parser)))
 
           ;; Save cache if possible and permitted
           (when (and
