@@ -1,6 +1,6 @@
 ;;; phps-mode-test.el --- Commons for tests -*- lexical-binding: t -*-
 
-;; Copyright (C) 2018-2022  Free Software Foundation, Inc.
+;; Copyright (C) 2018-2023  Free Software Foundation, Inc.
 
 
 ;;; Commentary:
@@ -18,7 +18,8 @@
   "The native tokens of PHP parser (if available).")
 
 (defmacro phps-mode-test--incremental-vs-intial-buffer (source &optional title &rest change)
-  "Set up test buffer with SOURCE, TITLE, apply CHANGE and compare incremental values with initial values."
+  "Set up test buffer with SOURCE, TITLE, apply CHANGE and
+compare incremental values with initial values."
   `(let ((test-buffer-incremental (generate-new-buffer "test-incremental"))
          (incremental-state)
          (incremental-state-stack)
@@ -68,7 +69,9 @@
      (setq incremental-states phps-mode-lex-analyzer--states)
      (setq incremental-tokens phps-mode-lex-analyzer--tokens)
      (setq incremental-imenu phps-mode-lex-analyzer--imenu)
-     (setq incremental-buffer (buffer-substring (point-min) (point-max)))
+     (save-restriction
+       (widen)
+       (setq incremental-buffer (buffer-substring (point-min) (point-max))))
      (setq incremental-bookkeeping (phps-mode-test--hash-to-list phps-mode-lex-analyzer--bookkeeping t))
      (setq incremental-nest-location-stack phps-mode-lex-analyzer--nest-location-stack)
 
@@ -168,7 +171,8 @@
        (message "\nPassed tests for '%s'\n" ,title))))
 
 (defun phps-mode-test--hash-to-list (hash-table &optional un-sorted)
-  "Return a list that represent the HASH-TABLE.  Each element is a list: (list key value), optionally UN-SORTED."
+  "Return a list that represent the HASH-TABLE.
+Each element is a list: (list key value), optionally UN-SORTED."
   (let (result)
     (if (hash-table-p hash-table)
         (progn
@@ -178,7 +182,12 @@
            hash-table)
           (if un-sorted
               (nreverse result)
-            (sort (nreverse result) (lambda (a b) (< (car a) (car b))))))
+            (sort
+             (nreverse result)
+             (lambda (a b)
+               (if (listp (car a))
+                   (< (car (car a)) (car (car b)))
+                 (< (car a) (car b)))))))
       nil)))
 
 (transient-mark-mode t)
