@@ -1,12 +1,12 @@
 ;;; phps-mode.el --- Major mode for PHP with code intelligence -*- lexical-binding: t -*-
 
-;; Copyright (C) 2018-2022  Free Software Foundation, Inc.
+;; Copyright (C) 2018-2023  Free Software Foundation, Inc.
 
 ;; Author: Christian Johansson <christian@cvj.se>
 ;; Maintainer: Christian Johansson <christian@cvj.se>
 ;; Created: 3 Mar 2018
-;; Modified: 21 May 2022
-;; Version: 0.4.21
+;; Modified: 8 Feb 2023
+;; Version: 0.4.37
 ;; Keywords: tools, convenience
 ;; URL: https://github.com/cjohansson/emacs-phps-mode
 
@@ -73,6 +73,22 @@
     map)
   "Keymap for `phps-mode'.")
 
+(defun phps-mode-error-notice ()
+  "Display error notices in echo are if any."
+  (when (and
+         phps-mode-lex-analyzer--error-start
+         phps-mode-lex-analyzer--error-message
+         (>= (point) phps-mode-lex-analyzer--error-start))
+    (let ((display-error))
+      (if phps-mode-lex-analyzer--error-end
+          (when (<= (point) phps-mode-lex-analyzer--error-end)
+            (setq display-error t))
+        (setq display-error t))
+      (when display-error
+        (message
+         "PHPs Error: %s"
+         phps-mode-lex-analyzer--error-message)))))
+
 ;;;###autoload
 (defun phps-mode-rescan-buffer ()
   "Re-scan buffer."
@@ -241,6 +257,12 @@
    0
    t)
 
+  ;; Support for error notices
+  (run-with-idle-timer
+   phps-mode-idle-interval
+   t
+   #'phps-mode-error-notice)
+
   ;; Initial run of lexer
   (phps-mode-lex-analyzer--re2c-run
    nil
